/*------------------------------------------------------------------------
 Module:        C:\Maison\CodeC\Echecs\CLSIMAGELIST.C
 Author:        Richard KIELBASA
 Project:       Echecs
 State:         Termin
 Creation Date: 19 12 2003
 Description:   Gestion des images

------------------------------------------------------------------------*/


#include <windows.h>
#include "echecs_rc.h"

#include "clsimagelist.h"
#include "frmColor.h"

#define NUM_ICONS 2

typedef struct _piecbmp {
	HBITMAP mask;
	HBITMAP outline;
}
PIECEBITMAP;

/*variables prives-----------------------------------------------------*/
static HIMAGELIST himlIcons;

/*images*/
static PIECEBITMAP pieces[7];
static HBRUSH brhBlackPiece = NULL;
static HBRUSH brhWhitePiece = NULL;
static HBRUSH brhShdBlackPiece = NULL;
static HBRUSH brhShdWhitePiece = NULL;

/*------------------------------------------------------------------------
Procedure:     Images_createListIcon ID:1
Purpose:
Input:
Output:
Errors:
------------------------------------------------------------------------*/
HIMAGELIST Images_createListIcon(HINSTANCE hinst)
{
	/*
	* declaration des variables locales
	* =================================
	*/
	HICON hicon;           /* handle to icon */

	/*
	* code executable
	* ===============
	*/

	/*Creer la liste d'image*/
	himlIcons = ImageList_Create(CX_ICON, CY_ICON, ILC_MASK, NUM_ICONS, 0);

	/*Charger les icones dans la liste d'images*/
	hicon = LoadIcon(hinst, MAKEINTRESOURCE(ICO_INACTIF));
	ImageList_AddIcon(himlIcons, hicon);
	DeleteObject(hicon);

	hicon = LoadIcon(hinst, MAKEINTRESOURCE(ICO_ACTIF));
	ImageList_AddIcon(himlIcons, hicon);
	DeleteObject(hicon);

	return himlIcons;
}


/*------------------------------------------------------------------------
Procedure:     Images_drawIcon ID:1
Purpose:

Input:
Output:
Errors:
------------------------------------------------------------------------*/
BOOL Images_drawIcon(HDC hDC, int indiceImage, int x0, int y0)
{
	/*
	* declaration des variables locales
	* =================================
	*/

	/*
	* code executable
	* ===============
	*/

	if (!ImageList_Draw(himlIcons, indiceImage, hDC, x0, y0, ILD_TRANSPARENT))
		return FALSE;

	return TRUE;
}

/*------------------------------------------------------------------------
Procedure:     Images_loadBmpPieces ID:1
Purpose:       Chargement indpendant de l'ordre de dclaration des
images
le fichier .rc
Input:
Output:
Errors:
------------------------------------------------------------------------*/
BOOL Images_loadBmpPieces(HINSTANCE hInst)
{
#define NBPIECES 6

	static int m[NBPIECES]= {
		BMP_PAWNM, BMP_KNIGHTM, BMP_BISHOPM, BMP_ROOKM, BMP_QUEENM, BMP_KINGM
	};
	static int o[NBPIECES]= {
		BMP_PAWNO, BMP_KNIGHTO, BMP_BISHOPO, BMP_ROOKO, BMP_QUEENO, BMP_KINGO
	};

	for ( int i = 0; i < NBPIECES; i++ )
	{
		pieces[i + 1].mask = LoadBitmap (hInst, MAKEINTRESOURCE(m[i]));
		pieces[i + 1].outline = LoadBitmap (hInst, MAKEINTRESOURCE(o[i]));
	}



	return TRUE;
}
/*------------------------------------------------------------------------
Procedure:     changePieceColor ID:1
Purpose:
Input:
Output:
Errors:
------------------------------------------------------------------------*/
BOOL Images_setPieceColor(void)
{
	COLORREF color;
	WORD r, g, b;


	if(brhBlackPiece != NULL)
		DeleteObject(brhBlackPiece);
	if(brhShdBlackPiece != NULL)
		DeleteObject(brhBlackPiece);

	brhBlackPiece = CreateSolidBrush(clrBlackPiece);
	b = GetBValue(clrBlackPiece)/2 ;
	g = GetGValue(clrBlackPiece)/2 ;
	r = GetRValue(clrBlackPiece)/2;
	color = RGB(r, g, b);
	brhShdBlackPiece = CreateSolidBrush(color);

	if(brhWhitePiece!= NULL)
		DeleteObject(brhWhitePiece);
	if(brhShdWhitePiece != NULL)
		DeleteObject(brhWhitePiece);
	brhWhitePiece = CreateSolidBrush(clrWhitePiece);
	b = GetBValue(clrWhitePiece)/2 ;
	g = GetGValue(clrWhitePiece)/2 ;
	r = GetRValue(clrWhitePiece)/2;
	color = RGB(r, g, b);
	brhShdWhitePiece = CreateSolidBrush(color);

	return TRUE;
}
/*------------------------------------------------------------------------
 Procedure:     Images_freeAll ID:1
 Purpose:
 Input:
 Output:
 Errors:
------------------------------------------------------------------------*/
BOOL Images_freeAll(void)
{
	ImageList_Destroy(himlIcons);

	/**/
	if(brhBlackPiece != NULL)
		DeleteObject(brhBlackPiece);
	if(brhShdBlackPiece != NULL)
		DeleteObject(brhBlackPiece);
	if(brhWhitePiece!= NULL)
		DeleteObject(brhWhitePiece);
	if(brhShdWhitePiece != NULL)
		DeleteObject(brhWhitePiece);

	for ( int i = 0; i < NBPIECES; i++ )
	{
		DeleteObject(pieces[i + 1].mask);
		DeleteObject(pieces[i + 1].outline );
	}


	return TRUE;
}

/*------------------------------------------------------------------------
Procedure:     Images_showPiece ID:1
Purpose:       Draw a piece in the specificed point
pieceBitmap is a structure with the handles for the
mask,
outline and piece.
Input:
Output:
Errors:
------------------------------------------------------------------------*/
void Images_showPiece ( HDC hDC, POINT *pptl, int size, int iPiece, COLORREF Color )
{
	HDC hMemDC;
	HBRUSH hOldBrush;
	PIECEBITMAP *pieceBitmap;
	int x0,y0;

	pieceBitmap = pieces + iPiece;
	x0 = pptl->x;
	y0 = pptl->y;

	hOldBrush = SelectObject ( hDC, GetStockObject (BLACK_BRUSH) );
	hMemDC = CreateCompatibleDC ( hDC);

	/* Write the mask to clear the space */
	SelectObject ( hMemDC, pieceBitmap->mask);
	//BitBlt ( hDC, pptl->x, pptl->y, PIECE_WIDTH, PIECE_HEIGHT, hMemDC, 0, 0, SRCAND);
	StretchBlt(
		hDC,	// handle of destination device context
		x0,	// x-coordinate of upper-left corner of dest. rect.
		y0,	// y-coordinate of upper-left corner of dest. rect.
		size,	// width of destination rectangle
		size,	// height of destination rectangle
		hMemDC,	// handle of source device context
		0,	// x-coordinate of upper-left corner of source rectangle
		0,	// y-coordinate of upper-left corner of source rectangle
		PIECE_WIDTH,	// width of source rectangle
		PIECE_HEIGHT,	// height of source rectangle
		SRCAND 	// raster operation code
		);


	/* Write out the piece with an OR : MERGEPAINT	PATPAINT	NOTSRCERASE SRCPAINT	?*/
	if(Color == clrBlackPiece)
		SelectObject ( hDC, brhBlackPiece);
	else
	    SelectObject ( hDC, brhWhitePiece);

	/*on redessine le masque mais en couleur*/
	//BitBlt ( hDC, pptl->x, pptl->y, PIECE_WIDTH, PIECE_HEIGHT, hMemDC, 0, 0,0xB80746L);
	StretchBlt(
		hDC,	// handle of destination device context
		x0,	// x-coordinate of upper-left corner of dest. rect.
		y0,	// y-coordinate of upper-left corner of dest. rect.
		size,	// width of destination rectangle
		size,	// height of destination rectangle
		hMemDC,	// handle of source device context
		0,	// x-coordinate of upper-left corner of source rectangle
		0,	// y-coordinate of upper-left corner of source rectangle
		PIECE_WIDTH,	// width of source rectangle
		PIECE_HEIGHT,	// height of source rectangle
		0xB80746L 	// raster operation code
		);

	/*calculer une couleur fonce*/
	if(Color == clrBlackPiece)
		SelectObject ( hDC, brhShdBlackPiece);
	else
	    SelectObject ( hDC, brhShdWhitePiece);


	/* The draw the outline */
	//SelectObject ( hDC, GetStockObject ( BLACK_BRUSH ) );
	SelectObject ( hMemDC, pieceBitmap->outline);
	//BitBlt ( hDC, pptl->x, pptl->y, PIECE_WIDTH, PIECE_HEIGHT, hMemDC, 0, 0, 0xB80746L);
	StretchBlt(
		hDC,	// handle of destination device context
		x0,	// x-coordinate of upper-left corner of dest. rect.
		y0,	// y-coordinate of upper-left corner of dest. rect.
		size,	// width of destination rectangle
		size,	// height of destination rectangle
		hMemDC,	// handle of source device context
		0,	// x-coordinate of upper-left corner of source rectangle
		0,	// y-coordinate of upper-left corner of source rectangle
		PIECE_WIDTH,	// width of source rectangle
		PIECE_HEIGHT,	// height of source rectangle
		0xB80746L 	// raster operation code
		);

	SelectObject ( hDC, hOldBrush );

	DeleteDC(hMemDC);
}


/*
C source for GNU CHESS

Revision: 26 12 2003
Amlioration de l'interface MS WINDOWS par Richard Kielbasa


Revision: 1990-09-30
Modified by Daryl Baker for use in MS WINDOWS environment

Copyright (C) 1986, 1987, 1988, 1989, 1990 Free Software Foundation, Inc.
Copyright (c) 1988, 1989, 1990  John Stanback

This file is part of CHESS.

CHESS is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY.  No author or distributor accepts responsibility to anyone for
the consequences of using it or for whether it serves any particular
purpose or works at all, unless he says so in writing.  Refer to the CHESS
General Public License for full details.

Everyone is granted permission to copy, modify and redistribute CHESS, but
only under the conditions described in the CHESS General Public License.
A copy of this license is supposed to have been given to you along with
CHESS so you can know your rights and responsibilities.  It should be in a
file named COPYING.  Among other things, the copyright notice and this
notice must be preserved on all copies.
*/
