/*------------------------------------------------------------------------
Module:        C:\DisqueD\CodeC\Echecs\book.c
Author:        Richard KIELBASA  (adaptateur)
Project:       Echecs
State:         Termin
Creation Date: 10 12 2003
Description:    C source for GNU CHESS
------------------------------------------------------------------------*/

#define NOATOM
#define NOCLIPBOARD
#define NOCREATESTRUCT
#define NOFONT
#define NOREGION
#define NOSOUND
#define NOWH
#define NOCOMM
#define NOKANJI

#include <windows.h>

#include <time.h>
#include <stdlib.h>
#include <stdio.h>

#include "gnuchess.h"
#include "search.h"
#include "mainechecs.h"
#include "Echecs_rc.h"

/*variables prives-------------------------------------------------*/
static unsigned int book_used = 0;
static char  * xBook;
static char *nomBook ="gnuchess.txt";
static GLOBALHANDLE hBook = 0;

/*------------------------------------------------------------------------
Procedure:     Book_Free ID:1
Purpose:
Input:
Output:
Errors:
------------------------------------------------------------------------*/
void Book_Free (void)
{
	if( !hBook) return;
	GlobalUnlock ( hBook );
	GlobalFree ( hBook);
	hBook = 0;
	book_used = 0;
}

/*------------------------------------------------------------------------
Procedure:     allocBook ID:1
Purpose:
Input:
Output:
Errors:
------------------------------------------------------------------------*/
static void  *allocBook ( unsigned int size )
{
	char  * temp;


	if ( book_used > 32*1024 ) return (void  *)0;
	temp = xBook+book_used;
	book_used += size;

	return temp;
}


/*------------------------------------------------------------------------
Procedure:     Book_getOpenings ID:1
Purpose:        Read in the Opening Book file and parse the algebraic notation for a move
into an unsigned integer format indicating the from and to square. Create
a linked list of opening lines of play, with entry->next pointing to the
next line and entry->move pointing to a chunk of memory containing the
moves. More Opening lines of up to 256 half moves may be added to
gnuchess.book.
Input:
Output:
Errors:
------------------------------------------------------------------------*/
void Book_getOpenings (HWND hWnd)

//#ifndef BOOK
//#define BOOK "gnuchess.boo"
//#endif /* BOOK */
{
	FILE *fd;
	int c, i, j, side;

	/* char buffr[2048]; */
	struct BookEntry   *entry;
	unsigned int mv,   *mp, tmp[100];
	/*
	char lpFile[_MAX_FNAME+_MAX_EXT+_MAX_DRIVE+_MAX_DIR+1];
	char sFname[_MAX_FNAME], sExt[_MAX_EXT], sDrive[_MAX_DRIVE], sDir[_MAX_DIR];

	GetModuleFileName ( (HINSTANCE)GetWindowLong(hWnd, GWL_HINSTANCE),
	lpFile, sizeof(lpFile) );

	_splitpath ( lpFile, sDrive, sDir, sFname, sExt);
	_makepath  ( lpFile, sDrive, sDir, "gnuchess", "txt");

	fd = fopen (lpFile, "r");
	*/

	fd = fopen (nomBook, "r");
	if (fd == NULL)
	{
		ShowMessage (hWnd, ITT_OBNF);
		return;
	}

	hBook = GlobalAlloc ( GMEM_MOVEABLE | GMEM_ZEROINIT, (long) (32*1024 * sizeof (char)) );
	if( hBook == NULL )
	{
		Book = NULL;
		ShowMessage (hWnd, ITT_OBAE);//Fehlermeldung
		fclose (fd);
		return;
	}
	xBook = (unsigned char  *) GlobalLock (hBook);


	/* setvbuf(fd,buffr,_IOFBF,2048); */
	Book = NULL;
	i = 0;
	side = white;
	while ((c = parse (fd, &mv, side)) >= 0)
		if (c == 1)
		{
			tmp[++i] = mv;
			side = otherside[side];
		}
		else if (c == 0 && i > 0)
		{
			entry = (struct BookEntry  *) allocBook ( sizeof (struct BookEntry));
			mp = (unsigned int  *) allocBook ( (i + 1) * sizeof (unsigned int));
			if ( (entry == 0 ) || (mp == 0) )
			{
				Book = NULL;
				ShowMessage (hWnd, ITT_OBAE);
				GlobalUnlock ( hBook );
				GlobalFree ( hBook);
				return;
			}
			entry->mv = mp;
			entry->next = Book;
			Book = entry;
			for (j = 1; j <= i; j++)
				*(mp++) = tmp[j];
			*mp = 0;
			i = 0;
			side = white;
		}
	fclose (fd);

}


/*------------------------------------------------------------------------
Procedure:     Book_opening ID:1
Purpose:       Go thru each of the opening lines of play and check for a match with the
current game listing. If a match occurs, generate a random number. If this
number is the largest generated so  then the next move in this line
becomes the current "candidate". After all lines are checked, the
candidate move is put at the top of the Tree[] array and will be played by
the program. Note that the program does not handle book transpositions.
Input:
Output:
Errors:
------------------------------------------------------------------------*/
void Book_opening (unsigned int *hint)

{
	int j, pnt;
	unsigned int m,  *mp;
	int r, r0;
	struct BookEntry  *p;

	srand ((unsigned int) time ((ULONG *) 0));
	r0 = m = 0;
	p = Book;
	while (p != NULL)
	{
		mp = p->mv;
		for (j = 1; j <= GameCnt; j++)
			if (GameList[j].gmove != *(mp++))
				break;
		if (j > GameCnt)
			if ((r = urand ()) > r0)
			{
				r0 = r;
				m = *mp;
				*hint = *(++mp);
			}
		p = p->next;
	}

	for (pnt = TrPnt[1]; pnt < TrPnt[2]; pnt++)
		if (((Tree[pnt].f << 8) | Tree[pnt].t) == m)
			Tree[pnt].score = 0;
	pick (TrPnt[1], TrPnt[2] - 1);
	if (Tree[TrPnt[1]].score < 0) {
		Book_Free ();
		Book = NULL;
	}
}

