/*------------------------------------------------------------------------
 Module:        C:\Maison\CodeC\Echecs\iofiles.c
 Author:        Adaptation : Richard KIELBASA
 Project:       Echecs
 State:         Termin
 Creation Date: 25 12 2003
 Description:   See at the end of file

------------------------------------------------------------------------*/


/*modifi par R.K.*/
#include <windows.h>
#include <shlobj.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <time.h>

#include "gnuchess.h"
#include "mainechecs.h"
#include "iofiles.h"
#include "display.h"
#include "search.h"
#include "echecs_rc.h"


/*procdures prives----------------------------------------------------*/
static char *obtenirDateHeure(void);

/*------------------------------------------------------------------------
 Procedure:     Iofiles_getDlgSaveOpen ID:1
 Purpose:
 Input:         dlgopen = TRUE ouvrir sinon enregistrer
 Output:
 Errors:
------------------------------------------------------------------------*/
BOOL Iofiles_getDlgSaveOpen (HWND hwndParent, char *sz_title, char *sz_ext,
	char *sz_desc, char *filename, int bufsize, BOOL dlgopen)
{
	OPENFILENAME    ofn;
	char           *sz_defilt = "Tous les fichiers (*.*)^*.*^";
	char            szFile[MAX_PATH];
	char            szFilter[147];
	char           *pch;
	char            szInitialDir[MAX_PATH];
	LPITEMIDLIST    ppidl;
	int				ii, ib, ie;
	BOOL			bMatched = TRUE;

	/*-------------------------------------------------------------------------*/
	/* Written by Olan Patrick Barnes                                          */
	/* April 7, 1999                                                           */
	/* Simplified by R.K.                                                      */
	/*-------------------------------------------------------------------------*/
	/* HISTORY:   03/24/98  Initial Write                             OPB      */
	/*            04/07/99  Fixed problems with save file             OPB      */
	/*                                                                         */
	/*-------------------------------------------------------------------------*/
	GetCurrentDirectory(MAX_PATH, szInitialDir);//js Pfad eintragen

	if ((sz_title == sz_desc) || (sz_ext == NULL))
		strcpy(szFilter, sz_defilt);
	else
	    {
		if (strlen(sz_ext) > 10)
			return (FALSE);

		if (strlen(sz_desc) > 100)
			return (FALSE);

		strcpy(szFilter, sz_desc);
		strcat(szFilter, " ( *.");
		strcat(szFilter, sz_ext);
		strcat(szFilter, " )^*.");
		strcat(szFilter, sz_ext);
		strcat(szFilter, "^");
		strcat(szFilter, sz_defilt);
	}

	for (pch = szFilter; *pch != 0; pch++)
		if (*pch == '^')
			*pch = '\0';

	szFile[0] = '\0';
	/* Set all structure members to zero. */
	memset(&ofn, 0, sizeof(OPENFILENAME));

	SHGetSpecialFolderLocation(hwndParent, CSIDL_DESKTOPDIRECTORY, &ppidl);
	//  SHGetPathFromIDList(ppidl, szInitialDir);
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.hwndOwner = hwndParent;
	ofn.lpstrFilter = szFilter;
	ofn.nFilterIndex = 0;
	ofn.lpstrFile = szFile;
	ofn.nMaxFile = sizeof(szFile);
	ofn.lpstrTitle = sz_title;
	ofn.lpstrInitialDir = szInitialDir;

	if (dlgopen == FILEOPEN)
		ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_EXPLORER;
	else
	    ofn.Flags = OFN_PATHMUSTEXIST | OFN_HIDEREADONLY | OFN_EXPLORER |OFN_OVERWRITEPROMPT;

	if (dlgopen == FILEOPEN)
	{
		if (GetOpenFileName(&ofn) != 0)
		{
			strncpy(filename, ofn.lpstrFile, bufsize);
			return (TRUE);
		}
		else
		    return (FALSE);
	}
	else
	    {
		ofn.Flags = OFN_OVERWRITEPROMPT;
		if (GetSaveFileName(&ofn) != 0)
		{
			strncpy(filename, ofn.lpstrFile, bufsize);

			ib = strlen(filename)-1;
			ie = strlen(sz_ext)-1;

			if (ib >= ie)
			{
				for (ii = 0; ii < (int)ie+1; ii++)
					if ((toupper(filename[ib-ii])) != (toupper(sz_ext[ie-ii])))
						bMatched = FALSE;

				if (bMatched)
					if (filename[ib-ii] != '.')
						bMatched = FALSE;
			}
			else
			    bMatched = FALSE;

			if ((ofn.nFilterIndex == 1) && (!bMatched))
				if ((strlen(filename)+strlen(sz_ext)+1) < MAX_PATH)
				{
					strcat(filename,".");
					strcat(filename,sz_ext);
				}

			return (TRUE);
		}
		else
		    return (FALSE);
	}
}


/*------------------------------------------------------------------------
Procedure:     Iofiles_listGame ID:1
Purpose:
Input:
Output:
Errors:
------------------------------------------------------------------------*/
void Iofiles_listGame (HWND hWnd, char *fname)
{
	FILE *fd;
	char chaine[MAXCHAINE];
	int i, f, t;

	if ( (fd = fopen (fname, "w")) == NULL)
	{
		ShowMessage (hWnd, ITT_LIST);
		return;
	}
	fprintf (fd, "<-----------------Echecs alias Chess---------------->\n");
	LoadString(hInst, ITT_LIST1, chaine, MAXCHAINE);
	fprintf (fd, chaine, obtenirDateHeure());
	fprintf (fd, "=====================================================\n\n");
	LoadString(hInst, ITT_LIST2, chaine, MAXCHAINE);
	fprintf (fd, chaine);
	fprintf (fd, chaine);
	fprintf (fd, "\n");

	for (i = 1; i <= GameCnt; i++)
	{
		f = GameList[i].gmove >> 8;
		t = (GameList[i].gmove & 0xFF);
		makeMoveString (f, t, false);
		//fprintf (fd, "%5s  %5d     %2d %7ld %5d", mvstr[0],
				fprintf (fd, "%5s  %5d     %2d %7ld %5d", Display_moveString,
			GameList[i].score, GameList[i].depth,
			GameList[i].nodes, GameList[i].time);

		if ((i % 2) == 0)
			fprintf (fd, "\n");
		else
		    fprintf (fd, "         ");

	}
	fprintf (fd, "\n\n");
	fprintf (fd, "====================fin de partie======================");

	fclose (fd);
}

/*------------------------------------------------------------------------
Procedure:     GetGame ID:1
Purpose:
Input:
Output:
Errors:
------------------------------------------------------------------------*/
void GetGame (HWND hWnd, char *fname)
{

	FILE *fd;
	int nbVal;
	int sq;

	struct GameRec tmp_rec;

	/*ouverture du fichier de la partie  charger*/
	fd = fopen (fname, "r");
	if (fd  == NULL)
	{
		ShowMessage (hWnd, ITT_LOADFAILED);
		return;
	}

	//hCursor = LoadCursor(NULL, IDC_WAIT	);
	//hCursor  = SetCursor(hCursor );
	/**/
	fscanf (fd, "%hd%hd%hd", &computer, &opponent, &Game50);
	/**/
	fscanf (fd, "%hd%hd", &castld[white], &castld[black]);
	/**/
	fscanf (fd, "%hd%hd", &TCflag, &OperatorTime);
	/**/
	fscanf (fd, "%ld%ld%hd%hd",
		&TimeControl.clock[white], &TimeControl.clock[black],
		&TimeControl.moves[white], &TimeControl.moves[black]);
	/*64 cases*/
	//printf("\n");
	for (sq = 0; sq < 64; sq++)
	{
		fscanf (fd, "%hd%hd%hd", &color[sq],  &board[sq], &Mvboard[sq]);
		//printf("%i ", board[sq]);
	}
	/**/
	GameCnt = 0;
	nbVal = 0;
	while (nbVal != EOF)
	{
		++GameCnt;
		//assert(GameCnt < SIZEGAMELIST);
		nbVal = fscanf (fd, "%hd%hd%hd%ld%hd%hd%hd", &tmp_rec.gmove,
			&tmp_rec.score, &tmp_rec.depth,
			&tmp_rec.nodes, &tmp_rec.time,
			&tmp_rec.piece, &tmp_rec.color);

		GameList[GameCnt] = tmp_rec;
		//printf("%i ", GameList[GameCnt].color);
	}

	GameCnt--;

	if (TimeControl.clock[white] > 0)
		TCflag = true;


	/**/
	fclose (fd);
	//SetCursor(hCursor );

	InitializeStats ();
	Sdepth = 0;
	UpdateDisplay (hWnd, 0, 0, 1, 0);


}

/*------------------------------------------------------------------------
Procedure:     SaveGame ID:1
Purpose:
Input:
Output:
Errors:
------------------------------------------------------------------------*/
void SaveGame (HWND hWnd, char *fname)
{
	FILE *fd;
	int sq, i;

	if(fname == NULL) return;
	/*fichier contenant le jeu de sortie*/
	fd = fopen (fname, "w");
	if (fd == NULL)
	{
		ShowMessage (hWnd, ITT_SAUVEGARDE);
		return;
	}
	/* enregistrement*/

	fprintf (fd, "%d %d %d\n", computer, opponent, Game50);
	fprintf (fd, "%d %d\n", castld[white], castld[black]);
	fprintf (fd, "%d %d\n", TCflag, OperatorTime);
	fprintf (fd, "%ld %ld %d %d\n",
		TimeControl.clock[white], TimeControl.clock[black],
		TimeControl.moves[white], TimeControl.moves[black]);
	for (sq = 0; sq < 64; sq++)
	{
		fprintf (fd, "%d %d %d\n", color[sq], board[sq], Mvboard[sq]);
		//printf("%i ", board[sq]);
	}
	for (i = 1; i <= GameCnt; i++)
	{
		fprintf (fd, "%d %d %d %ld %d %d %d\n",
			GameList[i].gmove, GameList[i].score, GameList[i].depth,
			GameList[i].nodes, GameList[i].time,
			GameList[i].piece, GameList[i].color);
		//printf("%i ", GameList[i].color);

	}

	fclose (fd);


}




/*------------------------------------------------------------------------
Procedure:     obtenirDateHeure ID:1
Purpose: A remplacer pour les anglophiles
Input:
Output:
Errors:
------------------------------------------------------------------------*/
static char *obtenirDateHeure(void)
{
	/*
	* declaration des variables locales
	* =================================
	*/

	struct  tm *timeptr;
	time_t ltime;

	static const char wday_name[7][3] = {
		"dim", "lun", "mar", "mer", "jeu", "ven", "sam"
	};

	static const char mon_name[12][3] = {
		"jan", "fev", "mar", "avr", "mai", "jun",
		"jui", "aou", "sep", "oct", "nov", "dec"
	};

	static char chaine[26];

	/*
	* code executable
	* ===============
	*/
	/* Get the time in seconds */
	time (&ltime);

	/* convert it to the structure tm */
	timeptr = localtime(&ltime);

	sprintf(chaine, "%.3s %3d %.3s %d %.2d:%.2d:%.2d",
		wday_name[timeptr->tm_wday],
		timeptr->tm_mday,
		mon_name[timeptr->tm_mon],
		1900 + timeptr->tm_year,
		timeptr->tm_hour,
		timeptr->tm_min,
		timeptr->tm_sec);

	return chaine;
}
